#include <errno.h>
#include <iostream>
#include <sstream>
#include <string>
#include <cstdlib>
#include <cstdio>
#include "player_common.h"

enum {
    Max_X = 13,
    Max_Y = 17,
};

FILE*
open_pipe( int fd, const char* mode )
{
    FILE* pipe = fdopen( fd, mode );

    if (!pipe)
        std::cerr << "Error accessing file descriptors" << std::endl;

    return pipe;
} // open_pipe


unsigned char
pipe_read_player( FILE* pipe )
{
    unsigned char player = 0;
    int numberOfReads = 0;

    do
        numberOfReads = fscanf( pipe, "%c\n", &player );
    while (numberOfReads == EOF && errno == EINTR);

    if (numberOfReads != 1)
    {
        std::cerr << "Error while reading the coordinates" << std::endl;
        return 0;
    }

    return player;
} // pipe_read_player


char**
pipe_read_board( FILE* pipe )
{
    char* buffer = NULL;
    char** board = new char*[Max_Y];
    size_t n;
    ssize_t read;

    do
        read = getline( &buffer, &n, pipe );
    while (read == -1);

    for (int y = 0; y < Max_Y; y++)
    {
        board[y] = new char[Max_X];

        for (int x = 0; x < Max_X; x++)
            board[y][x] = buffer[y * Max_X + x];
    }

    free( buffer );

    return board;
} // pipe_read_board


void
free_board( char** board )
{
    for (int y = 0; y < Max_Y; y++)
        free( board[y] );

    free( board );
} // free_board


void
print_board( char** board )
{
    std::cout << "0  1  2  3  4  5  6  7  8  9 10 11 12" << std::endl;
    for (unsigned int y = 0; y < Max_Y; y++)
    {
        for (unsigned int x = 0; x < Max_X; x++)
            std::cout << board[y][x] << "  ";

        std::cout << ": " << y << std::endl;
    }
} // print_board


std::string
get_next_move()
{
    int piece_x;
    int piece_y;
    std::stringstream ss( std::stringstream::in | std::stringstream::out );

    std::cout << std::endl
              << "Which piece do you want to move: " << std::endl
              << "   x: ";
    std::cin >> piece_x;

    std::cout << "    y: ";
    std::cin >> piece_y;

    ss << static_cast<unsigned int>(piece_x) << " " << static_cast<unsigned int>(piece_y) << " ";

    std::cout << std::endl
              << "Where do you want to move the piece at "
              << piece_x << "," << piece_y << " next? End your input with x: -1"
              << std::endl;

    do
    {
        std::cout << "   x: ";
        std::cin >> piece_x;

        if (piece_x == -1)
            break;

        std::cout << "   y: ";
        std::cin >> piece_y;

        ss << static_cast<unsigned int>(piece_x) << " " << static_cast<unsigned int>(piece_y) << " ";
    } while (true);

    ss << "\n";

    return ss.str();
} // get_next_move


void
pipe_send_move( FILE* pipe, std::string move )
{
    fprintf( pipe, "%s", move.c_str());

    /* Flush the buffer so the MCP actually receives our move. */
    fflush( pipe );
} // pipe_send_move


int
main( int argc, char* argv[] )
{
    FILE* pipeMCP_PLAYER = open_pipe( MCP_CHILD, "r" );
    FILE* pipePLAYER_MCP = open_pipe( CHILD_MCP, "w" );

    while (true)
    {
        /* Read input */
        char player = pipe_read_player( pipeMCP_PLAYER );
        std::cout << "[Player] I'm player " << player << std::endl;

        char** board = pipe_read_board( pipeMCP_PLAYER );

        /* Print the board */
        print_board( board );

        /* Ask where to place the piece. */
        std::string nextMove = get_next_move();
        std::cout << "[Player] Next move is: " << nextMove << std::endl;

        /* Output move */
        pipe_send_move( pipePLAYER_MCP, nextMove );
    }

    /* Cleanup */
    fclose( pipePLAYER_MCP );
    fclose( pipeMCP_PLAYER );

    /* Whenever we get here, we have an error */
    return 1;
} // main
